==> v1.0.0.tar.gz: platomix-gmetry-bi/platomix-gmetry-bi-api/src/main/java/org/springblade/cube/service/DynamicTableService.java <==
/**
 * 
 */
package org.springblade.cube.service;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.Timestamp;
import java.util.Date;
import java.util.List;
import java.util.Map;

import org.springblade.core.tool.utils.Func;
import org.springblade.cube.enums.ViewColumnTypeEnum;
import org.springblade.cube.mapper.DynamicTableMapper;
import org.springblade.cube.model.FileStructure;
import org.springblade.cube.utils.CubeConstants;
import org.springblade.cube.utils.CubeUtils;
import org.springblade.report.entity.ReportDatasource;
import org.springblade.report.support.DynamicDataSourceSupport;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import com.alibaba.druid.util.JdbcUtils;
import com.alibaba.fastjson2.JSONObject;
import com.baomidou.dynamic.datasource.annotation.DS;
import com.baomidou.mybatisplus.core.metadata.IPage;

import cn.hutool.core.date.DateUtil;
import cn.hutool.core.util.IdUtil;
import lombok.extern.slf4j.Slf4j;

/**
 * @author wookvn
 *
 *	动态创建表和插入数据
 */
@Slf4j
@Service
public class DynamicTableService {
	
	@Autowired
	private DynamicTableMapper dynamicTableMapper;
	
	/**
	 * 执行SQL
	 * @param ds
	 * @param sql
	 * @throws Exception 
	 */
	@DS("#ds.poolName")
	public void executeSql(ReportDatasource ds, String... sqls) {
		for(String sql : sqls) {
			dynamicTableMapper.executeSql(sql);
		}
	}
	
	/**
	 * 
	 * @param ds
	 * @param sql
	 * @return
	 */
	@DS("#ds.poolName")
	public List<Map<String, Object>> selectList(ReportDatasource ds, String sql) {
		return dynamicTableMapper.selectList(sql);
	}
	
	/**
	 * 
	 * @param ds
	 * @param sql
	 * @return
	 */
	@DS("#ds.poolName")
	public Map<String, Object> selectOne(ReportDatasource ds, String sql) {
		return dynamicTableMapper.selectOne(sql);
	}
	
	/**
	 * 
	 * @param page
	 * @param sql
	 * @return
	 */
	@DS("#ds.poolName")
	public List<Map<String, Object>> selectPageList(IPage<?> page, ReportDatasource ds, String sql) {
		return dynamicTableMapper.selectPageList(page, sql);
	}
	
	/**
	 * 
	 * @param page
	 * @param sql
	 * @return
	 */
	@DS("#ds.poolName")
	public IPage<Map<String, Object>> selectPage(IPage<?> page, ReportDatasource ds, String sql) {
		return dynamicTableMapper.selectPage(page, sql);
	}
	
	/**
	 * 批量入库
	 * @param ds
	 * @throws Exception
	 */
//	@DS("#ds.poolName")
	public void batchSave(ReportDatasource ds, String tableName, Long userId, FileStructure[] fileStructureArray, List<Map<Integer, String>> dataList) {
		PreparedStatement psStatement = null;
		Connection connection = null;
		try {
			String insertSql = CubeUtils.buildInsertSql(tableName, fileStructureArray);
			connection = DynamicDataSourceSupport.createConnection(ds);//dynamicRoutingDataSource.getConnection();
			psStatement = connection.prepareStatement(insertSql);
			connection.setAutoCommit(false);
			long startTime = System.currentTimeMillis();
			int size = dataList.size();
			int i = 1;
			for(Map<Integer, String> fileData : dataList) {
				for(int index = 1; index <= fileStructureArray.length; index++) {
					FileStructure fileStructure = fileStructureArray[index-1];
					//主键ID
					if(CubeConstants.COLUMN_USER_ID.equals(fileStructure.getColumnName())) {
						psStatement.setLong(index, userId);
						continue;
					}
					//USERID
					if(CubeConstants.COLUMN_PRIMARY_KEY.equals(fileStructure.getColumnName())) {
						psStatement.setLong(index, IdUtil.getSnowflakeNextId());
						continue;
					}
					//excel中其他数据，根据索引获取
					ViewColumnTypeEnum columnType = fileStructure.getColumnType();
					String value = fileData.get(fileStructure.getColumnIndex());
					if(columnType == ViewColumnTypeEnum.DATETIME) {
						java.sql.Timestamp dateValue = null;
						try {
							Date date = DateUtil.parse(value, CubeConstants.DATE_FORMAT_PATTERNS);
							dateValue = new Timestamp(date.getTime());
						} catch (Exception e) {}
						psStatement.setTimestamp(index, dateValue);
					}else if(columnType == ViewColumnTypeEnum.NUMBER) {
						try {
							value = value.replaceAll(CubeConstants.NUMBER_FORAMT_PATTERNS, "");
							psStatement.setDouble(index, Func.toDouble(value, 0D));
						} catch (Exception e) {
							psStatement.setDouble(index, 0D);
						}
					}else if(columnType == ViewColumnTypeEnum.BIGINT){
						try {
							psStatement.setLong(index, Func.toLong(value, 0L));
						} catch (Exception e) {
							psStatement.setLong(index, 0L);
						}
					}else{
						psStatement.setString(index, value);
					}
				}
				psStatement.addBatch();
				//达到批量入库的条数,开始入库操作
				if(i % ds.getDsTypeSyl().getBatchSize() == 0 || i == size) {
					log.info("数据建批完成|dataSize:{}, cost:{}ms", dataList.size(), System.currentTimeMillis() - startTime);
					psStatement.executeBatch();
					connection.commit();
					psStatement.clearBatch();
				}
				i++;
			}
//			psStatement.executeBatch();
//			connection.commit();
//			psStatement.clearBatch();
			long endTime = System.currentTimeMillis();
			log.info("批量入库成功|dataSize:{}, cost:{}ms", dataList.size(), endTime - startTime);
		} catch(Exception e) {
			throw new RuntimeException(e.getMessage(), e);
		} finally {
			JdbcUtils.close(psStatement);
			JdbcUtils.close(connection);
		}
	}
	
/*** ====================兼容老版本对外暴露的接口[后续在某个版本删除掉]=================**/	
	/**
	 * 
	 * @param ds
	 * @param sql
	 * @return
	 */
	@DS("#ds.poolName")
	@Deprecated
	public List<JSONObject> selectListForJSONObject(ReportDatasource ds, String sql) {
		return dynamicTableMapper.selectListForJSONObject(sql);
	}
	
	/**
	 * 
	 * @param ds
	 * @param sql
	 * @return
	 */
	@DS("#ds.poolName")
	@Deprecated
	public JSONObject selectOneForJSONObject(ReportDatasource ds, String sql) {
		return dynamicTableMapper.selectOneForJSONObject(sql);
	}
	
	/**
	 * 
	 * @param page
	 * @param sql
	 * @return
	 */
	@DS("#ds.poolName")
	@Deprecated
	public IPage<JSONObject> selectPageForJSONObject(IPage<?> page, ReportDatasource ds, String sql) {
		return dynamicTableMapper.selectPageForJSONObject(page, sql);
	}
}

==> v3.4.1.tar.gz: platomix-gmetry-bi/platomix-gmetry-bi-api/src/main/java/org/springblade/cube/service/DynamicTableService.java <==
/**
 * 
 */
package org.springblade.cube.service;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.Timestamp;
import java.util.Date;
import java.util.List;
import java.util.Map;

import org.springblade.core.tool.utils.Func;
import org.springblade.cube.mapper.DynamicTableMapper;
import org.springblade.cube.model.FileStructure;
import org.springblade.cube.utils.CubeConstants;
import org.springblade.cube.utils.CubeUtils;
import org.springblade.datasource.enums.ViewColumnTypeEnum;
import org.springblade.datasource.support.DynamicDataSourceSupport;
import org.springblade.report.entity.ReportDatasource;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import com.alibaba.druid.util.JdbcUtils;
import com.alibaba.fastjson2.JSONObject;
import com.baomidou.dynamic.datasource.annotation.DS;
import com.baomidou.mybatisplus.core.metadata.IPage;

import cn.hutool.core.date.DateUtil;
import cn.hutool.core.util.IdUtil;
import lombok.extern.slf4j.Slf4j;

/**
 * @author wookvn
 *
 *	动态创建表和插入数据
 */
@Slf4j
@Service
public class DynamicTableService {
	
	@Autowired
	private DynamicTableMapper dynamicTableMapper;
	
	/**
	 * 执行SQL
	 * @param ds
	 * @param sql
	 * @throws Exception 
	 */
	@DS("#ds.poolName")
	public void executeSql(ReportDatasource ds, String... sqls) {
		for(String sql : sqls) {
			dynamicTableMapper.executeSql(sql);
		}
	}
	
	/**
	 * 
	 * @param ds
	 * @param sql
	 * @return
	 */
	@DS("#ds.poolName")
	public List<Map<String, Object>> selectList(ReportDatasource ds, String sql) {
		return dynamicTableMapper.selectList(sql);
	}
	
	/**
	 * 
	 * @param ds
	 * @param sql
	 * @return
	 */
	@DS("#ds.poolName")
	public Map<String, Object> selectOne(ReportDatasource ds, String sql) {
		return dynamicTableMapper.selectOne(sql);
	}
	
	/**
	 * 
	 * @param page
	 * @param sql
	 * @return
	 */
	@DS("#ds.poolName")
	public List<Map<String, Object>> selectPageList(IPage<?> page, ReportDatasource ds, String sql) {
		return dynamicTableMapper.selectPageList(page, sql);
	}
	
	/**
	 * 
	 * @param page
	 * @param sql
	 * @return
	 */
	@DS("#ds.poolName")
	public IPage<Map<String, Object>> selectPage(IPage<?> page, ReportDatasource ds, String sql) {
		return dynamicTableMapper.selectPage(page, sql);
	}
	
	/**
	 * 批量入库
	 * @param ds
	 * @throws Exception
	 */
//	@DS("#ds.poolName")
	public void batchSave(ReportDatasource ds, String tableName, Long userId, FileStructure[] fileStructureArray, List<Map<Integer, String>> dataList) {
		PreparedStatement psStatement = null;
		Connection connection = null;
		try {
			String insertSql = CubeUtils.buildInsertSql(tableName, fileStructureArray);
			connection = DynamicDataSourceSupport.createConnection(ds);//dynamicRoutingDataSource.getConnection();
			psStatement = connection.prepareStatement(insertSql);
			connection.setAutoCommit(false);
			long startTime = System.currentTimeMillis();
			int size = dataList.size();
			int i = 1;
			for(Map<Integer, String> fileData : dataList) {
				for(int index = 1; index <= fileStructureArray.length; index++) {
					FileStructure fileStructure = fileStructureArray[index-1];
					//主键ID
					if(CubeConstants.COLUMN_USER_ID.equals(fileStructure.getColumnName())) {
						psStatement.setLong(index, userId);
						continue;
					}
					//USERID
					if(CubeConstants.COLUMN_PRIMARY_KEY.equals(fileStructure.getColumnName())) {
						psStatement.setLong(index, IdUtil.getSnowflakeNextId());
						continue;
					}
					//excel中其他数据，根据索引获取
					ViewColumnTypeEnum columnType = fileStructure.getColumnType();
					String value = fileData.get(fileStructure.getColumnIndex());
					if(columnType == ViewColumnTypeEnum.DATETIME) {
						java.sql.Timestamp dateValue = null;
						try {
							Date date = DateUtil.parse(value, CubeConstants.DATE_FORMAT_PATTERNS);
							dateValue = new Timestamp(date.getTime());
						} catch (Exception e) {}
						psStatement.setTimestamp(index, dateValue);
					}else if(columnType == ViewColumnTypeEnum.NUMBER) {
						try {
							value = value.replaceAll(CubeConstants.NUMBER_FORAMT_PATTERNS, "");
							psStatement.setDouble(index, Func.toDouble(value, 0D));
						} catch (Exception e) {
							psStatement.setDouble(index, 0D);
						}
					}else if(columnType == ViewColumnTypeEnum.BIGINT){
						try {
							psStatement.setLong(index, Func.toLong(value, 0L));
						} catch (Exception e) {
							psStatement.setLong(index, 0L);
						}
					}else{
						psStatement.setString(index, value);
					}
				}
				psStatement.addBatch();
				//达到批量入库的条数,开始入库操作
				if(i % ds.getDsTypeSyl().getBatchSize() == 0 || i == size) {
					log.info("数据建批完成|dataSize:{}, cost:{}ms", dataList.size(), System.currentTimeMillis() - startTime);
					psStatement.executeBatch();
					connection.commit();
					psStatement.clearBatch();
				}
				i++;
			}
//			psStatement.executeBatch();
//			connection.commit();
//			psStatement.clearBatch();
			long endTime = System.currentTimeMillis();
			log.info("批量入库成功|dataSize:{}, cost:{}ms", dataList.size(), endTime - startTime);
		} catch(Exception e) {
			throw new RuntimeException(e.getMessage(), e);
		} finally {
			JdbcUtils.close(psStatement);
			JdbcUtils.close(connection);
		}
	}
	
/*** ====================兼容老版本对外暴露的接口[后续在某个版本删除掉]=================**/	
	/**
	 * 
	 * @param ds
	 * @param sql
	 * @return
	 */
	@DS("#ds.poolName")
	@Deprecated
	public List<JSONObject> selectListForJSONObject(ReportDatasource ds, String sql) {
		return dynamicTableMapper.selectListForJSONObject(sql);
	}
	
	/**
	 * 
	 * @param ds
	 * @param sql
	 * @return
	 */
	@DS("#ds.poolName")
	@Deprecated
	public JSONObject selectOneForJSONObject(ReportDatasource ds, String sql) {
		return dynamicTableMapper.selectOneForJSONObject(sql);
	}
	
	/**
	 * 
	 * @param page
	 * @param sql
	 * @return
	 */
	@DS("#ds.poolName")
	@Deprecated
	public IPage<JSONObject> selectPageForJSONObject(IPage<?> page, ReportDatasource ds, String sql) {
		return dynamicTableMapper.selectPageForJSONObject(page, sql);
	}
}
